<?php

/**
 * @file
 * Dynamic style template.
 * Will be parsed with actual theme settings into a cacheable css file.
 */

// Mini helper.
$tgs = function($s) {
  return theme_get_setting($s, 'bilderblock');
};

?>
/**
 * Customized bilderblock styles, dynamically generated.
 *
 * @see style_static.css
 * @see style_dynamic.tpl.php
 */

/* === COMMON === */

/* Outer container (default state/menu opened) */
body > div#page > div#content-wrapper {
  margin-left: <?php print $tgs('menu_expanded_width'); ?>px;

<?php if ($content_max_width = $tgs('content_max_width')): ?>
  max-width: <?php print $content_max_width; ?>;
<?php endif; ?>
}
/* Outer container (menu is closed) */
body.bilderblock-menu-closed > div#page > div#content-wrapper {
  margin-left: <?php print $tgs('menu_collapsed_width') ?>px;
}

/* === END COMMON === */

/* === SIDEBAR === */
/* Menu and logo dimensions and positioning. */
.menu-expanded {
  /* Will be eventually overridden by config. */
  width: <?php print $tgs('menu_expanded_width'); ?>px;
}
.menu-collapsed {
  /* Will be eventually overridden by config. */
  width: <?php print $tgs('menu_collapsed_width'); ?>px;
}
div#menu-open {
  width: <?php print $tgs('menu_expanded_width') - $tgs('menu_collapsed_width'); ?>px;
}

/* === END SIDEBAR == */

/* === CONTENT === */

/* INNER CONTAINER */

/* Film strips */

/* Slides in general */
.teaser-slide-width,
div.teaser-slide a.images img {
  width: <?php print $tgs('teaser_slide_width'); ?>px;
}
.teaser-slide-height,
div.teaser-slide a.images img {
  height: <?php print $tgs('teaser_slide_height'); ?>px;
}
/* Information on the teaser side strips is expanded conditionally
   and then framed with a "has focus"-style border.
   To avoid text hopping, and to force proper/unchanged text wrapping
   for both states (cut-off and expanded), the actual text-holding element
   requires to have a fixed width. */
div.teaser-sidestrip-inner {
  width: <?php print ((int) $tgs('teaser_slide_width') - (BILDERBLOCK_SLIDE_SIDE_CONTENT_BORDER * 2)); ?>px;
}
<?php if ($op = $tgs('film_strip_hover')): ?>
div.teaser-sidestrip-inner {
  -moz-filter: brightness(<?php print $op; ?>%);
  -webkit-filter: brightness(<?php print $op; ?>%);
  -o-filter: brightness(<?php print $op; ?>%);
  filter: brightness(<?php print $op; ?>%);
}
<?php endif; ?>

/* EFFECT SETTINGS */
/* Overall filter animation speed. */
div#content-wrapper,
div.teaser-slide,
div.teaser-slide:after,
div.teaser-slide img {
  -moz-transition-duration: <?php print $tgs('fx_speed'); ?>ms;
  -webkit-transition-duration: <?php print $tgs('fx_speed'); ?>ms;
  -o-transition-duration: <?php print $tgs('fx_speed'); ?>ms;
  transition-duration: <?php print $tgs('fx_speed'); ?>ms;
}
<?php if (($sat = $tgs('fx_saturation')) != 100) : ?>
/* Decolorize filter. */
body.bilderblock-fx-saturation div.teaser-slide img {
  filter: saturate(<?php print $sat; ?>%);
}
<?php endif; ?>
/* Opacity */
body.bilderblock-fx-opacity div.teaser-slide {
  opacity: <?php print $tgs('fx_opacity') / 100; ?>;
}
body.bilderblock-fx-tint div.teaser-slide:after {
  opacity: <?php print $tgs('fx_tint') / 100; ?>;
}
