<?php

/**
 * @file
 * Advanced bilderblock theme settings.
 */

/**
 * Extend theme settings form with bilderblock specific items.
 */
function bilderblock_form_system_theme_settings_alter(&$form, $form_state) {

  $tgs = function($k) {
    return theme_get_setting($k, 'bilderblock');
  };

  // Vertical tabs style fix helper.
  // @see https://www.drupal.org/node/1143712
  $stylefix = array(
    'style' => 'width: 100%; max-width: 100%; width: auto;',
  );


  // Place vertical tabs directly after the preview area.
  $form['theme_settings_tabs'] = array(
    '#type' => 'vertical_tabs',
    '#weight' => -1,
  );

  // Move existing theme settings into tabs, too.
  $form['theme_settings']['#group'] = 'theme_settings_tabs';

  // Add "toggle breadcrumb" setting.
  $form['theme_settings']['toggle_breadcrumb'] = array(
    '#type' => 'checkbox',
    '#title' => t('Breadcrumb'),
    '#default_value' => $tgs('toggle_breadcrumb'),
  );

  // Prepare list of image fields.
  $source_fields = array();
  if ($fields = array_filter(field_info_field_map(), function($val) {
    return $val['type'] == 'image';
  })) {
    // Probably prepare image field input array.
    $source_fields['#tree'] = TRUE;
    $defaults = $tgs('source_fields');
    foreach (array_keys($fields) as $field_name) {
      $default = (array) @$defaults[$field_name];
      $default += array(
        'weight' => 0,
        'active' => 0,
      );
      $source_fields[$field_name] = array(
        '#type' => 'fieldset',
        '#title' => check_plain($field_name),
        '#weight' => $default['weight'],
        'active' => array(
          '#type' => 'checkbox',
          '#title' => t('Enabled'),
          '#default_value' => $default['active'],
        ),
        'weight' => array(
          '#title' => t('Weight'),
          '#type' => 'weight',
          '#default_value' => $default['weight'],
        ),
      );
    }
  }

  // Add layout settings.
  $form['theme_settings']['layout'] = array(
    '#weight' => 9,
    '#type' => 'fieldset',
    '#title' => t('Layout'),
    '#description' => t('Configure various format dimensions.'),
    '#collapsible' => TRUE,
    '#group' => 'theme_settings_tabs',
    'menu_dimensions' => array(
      '#type' => 'fieldset',
      '#title' => t('Menu'),
      '#description' => t('Settings related to the menu sidebar.'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      'menu_collapsed_width' => array(
        '#type' => 'textfield',
        '#size' => 5,
        '#field_suffix' => t('px', array(), array('context' => 'CSS pixels')),
        '#title' => t('Collapsed width'),
        '#description' => t('Width of the outer menu side-strip (less than some 20 may cause problems).'),
        '#default_value' => $tgs('menu_collapsed_width'),
        '#attributes' => $stylefix,
      ),
      'menu_expanded_width' => array(
        '#type' => 'textfield',
        '#size' => 5,
        '#field_suffix' => t('px', array(), array('context' => 'CSS pixels')),
        '#title' => t('Expanded width'),
        '#description' => t('Overall maximum sidebar size when expanded (including the "collapsed" width).'),
        '#default_value' => $tgs('menu_expanded_width'),
        '#attributes' => $stylefix,
      ),
      'menu_default_closed' => array(
        '#type' => 'checkbox',
        '#title' => t('Collapse menu by default'),
        '#description' => t('If checked, the menu sidebar will be hidden by default. It will expand whenever the lean side strip is hovered. With Javascript enabled, users can also toggle the menu state persistently, the state will be remembered in a cookie.'),
        '#default_value' => $tgs('fx_hover_clear'),
      ),
      'menu_icon' => array (
        '#type' => 'fieldset',
        '#title' => t('Menu icon settings'),
        '#description' => t('If toggled on, the following menu icon will be displayed.'),
        '#attributes' => array (
          'class' => array('theme-settings-bottom'),
        ),
        'default_menu_icon' => array (
          '#type' => 'checkbox',
          '#title' => t('Use the default menu icon'),
          '#default_value' => $tgs('default_menu_icon'),
          '#tree' => FALSE,
          '#description' => t('Check here if you want the theme to use the menu icon supplied with it.'),
        ),
        'settings' => array (
          '#type' => 'container',
          '#states' => array (
            'invisible' => array (
              'input[name="default_menu_icon"]' => array (
                'checked' => true,
              ),
            ),
          ),
          'menu_icon_path' => array (
            '#type' => 'textfield',
            '#title' => t('Path to custom menu icon'),
            '#description' => t('The path to the file you would like to use as your menu icon instead of the default. Should not exceed 40x40 pixels.'),
            '#default_value' => $tgs('menu_icon_path'),
          ),
          'menu_icon_upload' => array (
            '#type' => 'file',
            '#title' => t('Upload menu icon'),
            '#maxlength' => 40,
            '#description' => t("If you don't have direct file access to the server, use this field to upload your menu icon."),
          ),
        ),
      ),
    ),
    'slide_dimensions' => array(
      '#type' => 'fieldset',
      '#title' => t('Slides'),
      '#description' => t('Currently, slides are forced into equal dimensions. These ideally correspond with the actual image style in use.'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      'teaser_slide_height' => array(
        '#type' => 'textfield',
        '#size' => 5,
        '#field_suffix' => t('px', array(), array('context' => 'CSS pixels')),
        '#title' => t('Height'),
        '#default_value' => $tgs('teaser_slide_height'),
        '#attributes' => $stylefix,
      ),
      'teaser_slide_width' => array(
        '#type' => 'textfield',
        '#size' => 5,
        '#field_suffix' => t('px', array(), array('context' => 'CSS pixels')),
        '#title' => t('Width'),
        '#default_value' => $tgs('teaser_slide_width'),
        '#attributes' => $stylefix,
      ),
    ),
    'content_dimensions' => array(
      '#type' => 'fieldset',
      '#title' => t('Content'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      'content_max_width' => array(
        '#type' => 'textfield',
        '#size' => 5,
        '#title' => t('Content border'),
        '#description' => t('Optionally limit the actual content width by a valid CSS value.'),
        '#field_prefix' => 'max-width: ',
        '#field_suffix' => ';',
        '#default_value' => $tgs('content_max_width'),
        '#attributes' => $stylefix,
      ),
      'toggle_post_image_border' => array(
        '#type' => 'checkbox',
        '#title' => t('Exposed image border'),
        '#description' => t('In full node display, the exposed image is presented outside all other standard blocks. Enable this setting to have it displayed with a border of the color set as above.'),
        '#default_value' => $tgs('toggle_post_image_border'),
      ),
    ),
  );

  // Add fx settings.
  $form['theme_settings']['fx'] = array(
    '#weight' => 10,
    '#type' => 'fieldset',
    '#title' => t('Film strip effects'),
    '#description' => t('Additional client-side image processing applied to the "film strip" areas containing teaser slides.'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#group' => 'theme_settings_tabs',
    'film_strip' => array(
      '#type' => 'fieldset',
      '#title' => t('Side strips'),
      '#description' => t('Settings related to the film strip side areas, containing node links and other data.'),
      'film_strip_hover' => array(
        '#type' => 'select',
        '#options' => _bilderblock_options_percent(5, 100, 100),
        '#title' => t('Side-strip opacity'),
        '#description' => t("If set to a value other than 100, each slide's side-strip will be displayed with this opacity unless being hovered."),
        '#default_value' => $tgs('film_strip_hover'),
      ),
    ),
    'slides' => array(
      '#type' => 'fieldset',
      '#title' => t('Slides'),
      '#description' => t('Settings related to the actual teaser slide images (will be without effect in certain/older browsers).'),
      'fx_saturation' => array(
        '#type' => 'select',
        '#options' => _bilderblock_options_percent(10, 300, 100),
        '#title' => t('Saturation'),
        '#description' => t("Increase or reduce the original image's saturation. 100% equals the original value, 0 results in greyscale."),
        '#default_value' => $tgs('fx_saturation'),
      ),
      'fx_tint' => array(
        '#type' => 'select',
        '#options' => _bilderblock_options_percent(),
        '#title' => t('Tint'),
        '#description' => t('Whether to add a color overlay to each slide, in the film strip text color, and how opaque this overlay is supposed to be.'),
        '#default_value' => $tgs('fx_tint'),
      ),
      'fx_opacity' => array(
        '#type' => 'select',
        '#options' => _bilderblock_options_percent(5, 100, 100),
        '#title' => t('Opacity'),
        '#description' => t('If set to a value other than 100, slides will be displayed with this opacity unless being hovered. Can be used for e. g. a lightbox effect in combination with hovering.'),
        '#default_value' => $tgs('fx_opacity'),
      ),
      'hover' => array(
        '#type' => 'fieldset',
        '#title' => t('Hover effects'),
        '#description' => t('Instead of solely applying static effects to slide images, the "hover" event can be used to clear all effects from a slide.'),
        'fx_hover_clear' => array(
          '#type' => 'checkbox',
          '#title' => t('Hover clears effects'),
          '#description' => t('If checked, all filters and effects applied to a slide image will be removed when the element is hovered. Also, any filter/effect will be disabled on devices with limited hover capabilities, such as touch.'),
          '#default_value' => $tgs('fx_hover_clear'),
        ),
        'fx_speed' => array(
          '#type' => 'textfield',
          '#size' => 5,
          '#field_suffix' => t('ms', array(), array('context' => 'milliseconds')),
          '#title' => t('Effects duration'),
          '#description' => t('Enter an integer value of zero or more milliseconds.'),
          '#default_value' => $tgs('fx_speed'),
          '#attributes' => $stylefix,
        ),
      ),
    ),
  );

  // Add label settings.
  $labels = $tgs('labels');
  $form['theme_settings']['labels'] = array(
    '#weight' => 10,
    '#type' => 'fieldset',
    '#title' => t('Film strip labels'),
    '#description' => t('Customize label for node metadata on teaser slides.'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#group' => 'theme_settings_tabs',
    '#tree' => TRUE,
    'by' => array(
      '#type' => 'textfield',
      '#title' => t('By'),
      '#description' => t('This text will be used as a label for the "submitted" property on each node teaser ("slide"). Leave empty for no label.'),
      '#maxlength' => 32,
      '#default_value' => $labels['by'],
    ),
    'date' => array(
      '#type' => 'textfield',
      '#title' => t('Date'),
      '#description' => t('This text will be used as a label for the "date" property on each node teaser ("slide"). Leave empty for no label.'),
      '#maxlength' => 32,
      '#default_value' => $labels['date'],
    ),
  );

  // Add image source mapping.
  $form['theme_settings']['source_fields'] = array(
    '#weight' => 11,
    '#type' => 'fieldset',
    '#title' => t('Image sources'),
    '#description' => t('Define which fields are used as "teaser slide"/"exposed image". To use a field as a source, enable it and optionally set its weight.'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#group' => 'theme_settings_tabs',
    '#tree' => TRUE,
  ) + $source_fields;


  // Add submit callback to regenerate dynamic css.
  $form['#validate'][] = 'bilderblock_theme_settings_validate';
  $form['#submit'][] = 'bilderblock_theme_settings_submit';
}

/**
 * Theme settings submit callback.
 */
function bilderblock_theme_settings_submit(&$form, $form_state) {
  // Regenerate dynamic css.
  require_once drupal_get_path('theme', 'bilderblock') . '/template.php';

  // Regenerate dymamic styles, properties, variables etc.
  bilderblock_dynamic_css(TRUE);
  bilderblock_fallback_image();
}

/**
 * Theme settings validation callback.
 */
function bilderblock_theme_settings_validate(&$form, &$form_state) {
  $v = $form_state['values'];
  // FX speed must be (int) >= 0
  if ($v['fx_speed'] != abs((int) $v['fx_speed'])) {
    form_set_error('fx_speed', t('%value is not a valid effects duration in milliseconds.', array(
      '%value' => $v['fx_speed'],
    )));
  }
  // Width must be (int) >= 0
  if ($v['menu_collapsed_width'] != abs((int) $v['menu_collapsed_width'])) {
    form_set_error('menu_collapsed_width', t('%value is not a valid value for the collapsed menu width.', array(
      '%value' => $v['menu_collapsed_width'],
    )));
  }
  // Width must be (int) > 0
  if ($v['menu_expanded_width'] != (max(abs((int) $v['menu_expanded_width']), 1))) {
    form_set_error('menu_expanded_width', t('%value is not a valid value for the expanded menu width.', array(
      '%value' => $v['menu_expanded_width'],
    )));
  }
  // Width must be (int) > 0
  if ($v['teaser_slide_width'] != (max(abs((int) $v['teaser_slide_width']), 1))) {
    form_set_error('teaser_slide_width', t('%value is not a valid value for the teaser slide width.', array(
      '%value' => $v['teaser_slide_width'],
    )));
  }
  // Width must be (int) > 0
  if ($v['teaser_slide_height'] != (max(abs((int) $v['teaser_slide_height']), 1))) {
    form_set_error('teaser_slide_height', t('%value is not a valid value for the teaser slide height.', array(
      '%value' => $v['teaser_slide_height'],
    )));
  }
  $validators = array('file_validate_is_image' => array());

  // Check for a new uploaded logo.
  $file = file_save_upload('menu_icon_upload', $validators);
  if (isset($file)) {
    // File upload was attempted.
    if ($file) {
      // Put the temporary file in form_values so we can save it on submit.
      $form_state['values']['menu_icon_upload'] = $file;
      $filename = file_unmanaged_copy($file->uri);
      $form_state['values']['menu_icon_path'] = _system_theme_settings_validate_path($filename);
      $form_state['values']['default_menu_icon'] = 0;
    }
    else {
      // File upload failed.
      form_set_error('menu_icon_upload', t('The menu icon could not be uploaded.'));
    }
  }
}

/**
 * Generate a percents option array for form elements.
 *
 * @param int $step
 *   (Optional) Positive integer defining the step width. Minimum is 1.
 *   Defaults to 5.
 * @param int $end
 *   (Optional) Positive integer defining the upper scale end.
 *   Defaults to 100.
 * @param mixed $disable_value
 *   (Optional) value in the range from 0 to 100 that will be replaced by
 *   $disable_label in the options label. Set to FALSE to ignore.
 *   Defaults to 0.
 * @param string $disable_label
 *   (Optional) Label that will conditionally replace $disable_value.
 *
 * @return array
 *   Options array.
 */
function _bilderblock_options_percent($step = 5, $end = 100, $disable_value = 0, $disable_label = '---') {
  $start = 0;
  // Steps must be positive integers.
  $step = max(abs((int) $step), 1);
  // Avoid deadlock and force at least one value.
  if ($end <= $start) {
    $end = $start;
  }
  // Init resut array and counter.
  $count = $start;
  $options = array();
  while ($count < $end) {
    $options[$count] = $count;
    $count += $step;
  }
  // Make sure final value is included regardless of step width.
  $options[$end] = $end;
  // Format resulting array for form API.
  foreach ($options as $k => $v) {
    $options[$k] = $disable_value !== FALSE && $v == $disable_value ?
      $disable_label : format_string('!p %', array('!p' => $v));
  };
  return $options;
}
